"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const useUpdate_1 = require("./useUpdate");
const useGetSetState = (initialState = {}) => {
    if (process.env.NODE_ENV !== 'production') {
        if (typeof initialState !== 'object') {
            throw new TypeError('useGetSetState initial state must be an object.');
        }
    }
    const update = useUpdate_1.default();
    const state = react_1.useRef(Object.assign({}, initialState));
    const get = react_1.useCallback(() => state.current, []);
    const set = react_1.useCallback((patch) => {
        if (!patch)
            return;
        if (process.env.NODE_ENV !== 'production') {
            if (typeof patch !== 'object') {
                throw new TypeError('useGetSetState setter patch must be an object.');
            }
        }
        Object.assign(state.current, patch);
        update();
    }, []);
    return [get, set];
};
exports.default = useGetSetState;
