"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const isClient = typeof window === 'object';
const useLocalStorage = (key, initialValue, raw) => {
    if (!isClient) {
        return [initialValue, () => { }];
    }
    const [state, setState] = react_1.useState(() => {
        try {
            const localStorageValue = localStorage.getItem(key);
            if (typeof localStorageValue !== 'string') {
                localStorage.setItem(key, raw ? String(initialValue) : JSON.stringify(initialValue));
                return initialValue;
            }
            else {
                return raw ? localStorageValue : JSON.parse(localStorageValue || 'null');
            }
        }
        catch (_a) {
            // If user is in private mode or has storage restriction
            // localStorage can throw. JSON.parse and JSON.stringify
            // can throw, too.
            return initialValue;
        }
    });
    react_1.useEffect(() => {
        try {
            const serializedState = raw ? String(state) : JSON.stringify(state);
            localStorage.setItem(key, serializedState);
        }
        catch (_a) {
            // If user is in private mode or has storage restriction
            // localStorage can throw. Also JSON.stringify can throw.
        }
    });
    return [state, setState];
};
exports.default = useLocalStorage;
