"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const isClient = typeof window === 'object';
const useWindowSize = (initialWidth = Infinity, initialHeight = Infinity) => {
    const [state, setState] = react_1.useState({
        width: isClient ? window.innerWidth : initialWidth,
        height: isClient ? window.innerHeight : initialHeight,
    });
    react_1.useEffect(() => {
        if (isClient) {
            const handler = () => {
                setState({
                    width: window.innerWidth,
                    height: window.innerHeight
                });
            };
            window.addEventListener('resize', handler);
            return () => window.removeEventListener('resize', handler);
        }
        else {
            return undefined;
        }
    }, []);
    return state;
};
exports.default = useWindowSize;
